from path import Path
import jujuresources
from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core import unitdata
from jujubigdata import utils
from jujubigdata.relations import HBase
#from phoenix_util import get_zookeeper_servers



# Extended status support
# We call update_blocked_status from the "requires" section of our service
# block, so be sure to return True. Otherwise, we'll block the "requires"
# and never move on to callbacks. The other status update methods are called
# from the "callbacks" section and therefore don't need to return True.
def update_blocked_status():
    if unitdata.kv().get('charm.active', False):
        return True
    dep = HBase()
    if not dep.connected_units():
        hookenv.status_set('blocked', 'Waiting for relation to apache-hbase')
    elif not dep.is_ready():
        hookenv.status_set('waiting', 'Waiting for Apache HBase to become ready')
    return True


def update_working_status():
    if unitdata.kv().get('charm.active', False):
        hookenv.status_set('maintenance', 'Updating configuration')
        return
    hookenv.status_set('maintenance', 'Setting up Apache Phoenix')


def update_active_status():
    unitdata.kv().set('charm.active', True)
    hookenv.status_set('active', 'Ready')


# Main Phoenix class for callbacks
class Phoenix(object):
    def __init__(self, dist_config):
        self.dist_config = dist_config
        self.resources = {
            'phoenix': 'phoenix-%s' % host.cpu_arch(),
        }
        self.verify_resources = utils.verify_resources(*self.resources.values())

    def is_installed(self):
        return unitdata.kv().get('phoenix.installed')

    def install(self, force=False):
        
        if not force and self.is_installed():
            return
        self.dist_config.add_dirs()
        jujuresources.install(self.resources['phoenix'],
                              destination=self.dist_config.path('phoenix'),
                              skip_top_level=True)
        self.setup_phoenix_env()
        unitdata.kv().set('phoenix.installed', True)
        unitdata.kv().flush(True)
        
    def setup_phoenix_env(self):
        phoenix_bin = self.dist_config.path('phoenix')  / 'bin'
        # update environment variables
        with utils.environment_edit_in_place('/etc/environment') as env:
            if phoenix_bin not in env['PATH']:
                env['PATH'] = ':'.join([env['PATH'], phoenix_bin])
        
    def setup_phoenix_hbase(self):
        import sys
        import glob, os, shutil
        #if not unitdata.kv().get('hbase.installed'):
        phoenix_lib = self.dist_config.path('phoenix') 
        hbase_lib = self.dist_config.path('hbase') / 'lib'
        if not os.path.isdir(hbase_lib):
            print ("This node is not an hbase managed node - just return - valid only for Phoenix client")
            return
            
        files = glob.iglob(os.path.join(phoenix_lib, "*.jar"))
        for phoenix_jar_file in files:
            if os.path.isfile(phoenix_jar_file):
                shutil.copy2(phoenix_jar_file, hbase_lib)
        
        
       
    def start(self):
        print("re-start HBase daemons ......")
        node_type = ""
        if utils.jps("HMaster"): 
            node_type="master"
        elif utils.jps("HRegionServer"):
            node_type="regionserver"
        else:
            print ("This node is not an hbase managed node ... valid for Phoenix Client")
            return
        hbase_home = self.dist_config.path('hbase')
        utils.run_as('ubuntu', '{}/bin/hbase-daemon.sh'.format(hbase_home),'restart', node_type)

    def stop(self):
        print("do nothing ......")
       
        

    def cleanup(self):
        self.dist_config.remove_dirs()
        unitdata.kv().set('phoenix.installed', False)
        unitdata.kv().flush(True)
        
       
